// =============================================================================
// This file is part of boinctui.
// http://boinctui.googlecode.com
// Copyright (C) 2012,2013 Sergey Suslov
//
// boinctui is free software; you can redistribute it and/or modify it  under
// the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// boinctui is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details
// <http://www.gnu.org/licenses/>.
// =============================================================================

// * для ведения логов

#ifndef KCLOG_H_
#define KCLOG_H_

#include <stdio.h>

/// открывает лог файл на запись
void kLogOpen(const char* filename = NULL);

/// закрывает лог файл
void kLogClose();

/// вывести информацию в лог 
void kLogPrintf(const char* fmt, ...);

#endif /*KCLOG_H_*/
